<?php
namespace Teknisa\DF\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Doctrine\ORM\EntityManager;
use Teknisa\DF\Services\Ger57000TabelaDoSimplesNacionalService;

class Ger57000TabelaDoSimplesNacionalController extends \Teknisa\DF\Controllers\TeknisaCrud {
    
    protected $entityManager;
    protected $dataSourceName;
    protected $parameterBagColumns; 
    protected $crudService;
    protected $service;
     
    public function __construct($crudService, $parameterBag, Ger57000TabelaDoSimplesNacionalService $service, $entityManager) {
        $this->crudService = $crudService;
        $this->service = $service;
        $this->entityManager = $entityManager;
    }
    
    public function deleteTabelaDeReferencia(DTO\Request $request, DTO\Response $response){
        try {
            $rows = $request->getRow();
            $tabela = $rows['CDSIMPLESNACIONAL'];
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $this->service->deleteTabelaDeReferencia($tabela);
            
            $this->entityManager->flush();
            $connection->commit();
            return true;
        }
        catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            $connection->rollback();
        } 
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $rota = substr($request->getRoutePath(), 17, strlen($request->getRoutePath()) - 17);
            $dataSourceNameToSave = $rota;
    
            switch ($rota) {
            case 'saveCodigoFiscalDeOperacao':
                $dataSourceName = 'cfopsimplesnac';
                $this->service->validaIntervaloCfop($rows);
                break;
            case 'saveTipoDeOperacao':
                $dataSourceName = 'tpopersimplesnac'; 
                $this->service->validaIntervaloTipoOperacao($rows);
                break;
            case 'saveFaixaDeValores':
                $dataSourceName = 'faixasimplesnac';
                break;
            }
            //unset($rows[0]['DSCFOPPADRAO']);
            //unset($rows[0]['NMTIPOOPER']);
            //$this->RowsTosave = $rows;
            $this->dataSourceNameToSave = $dataSourceName;
            $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
            $this->crudService->save($dataSourceName, $rows);
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }    
    }
    
    public function saveFaixa(Request\DataSet $request, Response $response) {
        try {
            $rowFaixa   = $request->getDataSet()->getRows()[0];
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $allDataSet = $request->getDataSet()->getRows();
            $prxFaixa   = $this->service->getPrxFaixa($allDataSet[0]->get('CDSIMPLESNACIONAL'));
            
            // Caso esteja editando os registro apaga todas as faixas , para inserir novamente
            //para evitar que uma faixa apagada pelo usuario continue salvo.
            if(!$allDataSet[0]['__is_new']){
                $this->service->deleteImpostosFaixa($allDataSet[0]);
                $this->entityManager->flush();    
                
                // Percorre todos os impostos para adiconar-los novamente
                for ($i=1; $i<count($allDataSet); $i++){
                    $allDataSet[$i]->set('__is_new', true);
                    $allDataSet[$i]->set('CDFAIXA', $allDataSet[0]['CDFAIXA']);
                }
            }
        
            //Caso esteja inserindo uma nova faixa, pega o sequencial CDFAIXA.
            if($allDataSet[0]['__is_new']){
                foreach($allDataSet as &$rowToSave) {
                    $rowToSave->set('CDFAIXA', $prxFaixa);
                }    
            }
            
            $dataSet = new DataSet('faixasimplesnac', array($rowFaixa));
            $requestSalvar = new Request\DataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
            $this->service->saveFaixa($requestSalvar);
            $this->entityManager->flush();
            
            unset($allDataSet[0]);
            
            $dataSetImpFaixa = new DataSet('impsimplesfaixa', $allDataSet);
            $requestImpFaixa = new Request\DataSet($dataSetImpFaixa, $request->getMethod(),$request->getRoutePath(), $request->getUserId());

            $this->service->saveImpFaixa($requestImpFaixa);
            $this->entityManager->flush();
            
            $connection->commit();
        }
        catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            $connection->rollback();
        }
    }
    
    public function deleteFaixa(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $this->service->deleteFaixa($rows);
            
            $this->entityManager->flush();
            $connection->commit();
            return true;
        }
        catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            $connection->rollback();
        } 
    }
    
    public function  saveTabelaSimplesNacional(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            if(!$row['__is_new'])
                $this->service->validaEdicaoTabelaSimplesNacional($row);
            
            $this->dataSourceName = 'simplesnacional';
            parent::save($request, $response);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    
}