<?php

namespace Teknisa\DF\Factories;

use Doctrine\ORM\EntityManager;

class ParametrizacaoFiscalCentralizadaFactory
{
    private $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function consultarErrosRegra($cdregrafisc)
    {
        $sql = <<<SQL
        SELECT DISTINCT
           ERRO.CDIMPOSTO,
           I.NMIMPOSTO,
           ERRO.CDIMPOSTONFE,
           INFE.SGIMPOSTONFE,
           ERRO.CDSITUTRIB,
           P.DSSITUTRIB,
           ERRO.CDFISCAL,
           ERRO.VRALIQUOTA,
           ERRO.CDFISCOPER,
           ERRO.IDCADPADRAO,
           TO_CHAR(ERRO.IDERRO) as IDERRO
        FROM TABLE(CONSULTA_ERRO_REGRAFISCAL(:P_CDREGRAFISC)) ERRO
        LEFT JOIN IMPOSTO I
        ON I.CDIMPOSTO = ERRO.CDIMPOSTO
        LEFT JOIN IMPOSTONFE INFE
        ON I.CDIMPOSTONFE = INFE.CDIMPOSTONFE
        LEFT JOIN PARAMCST P
        ON P.CDSITUTRIB = ERRO.CDSITUTRIB
        ORDER BY ERRO.IDERRO
SQL;

        $params = array(
            "P_CDREGRAFISC" => $cdregrafisc
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function consultaRepetida($cdempresa,$identrsaidop,$cdtipooper,$cdregrafisc){
        
        $sql = <<<SQL
        SELECT DISTINCT
            V.CDREGRAFISC,
            V.NRVINCFISCREGRA,
            V.CDEMPRESA,
            E.NMEMPRESA,
            V.IDENTRSAIDOP,
            V.CDTIPOOPER,
            T.NMTIPOOPER,
            V.CDINSCESTA,
            I.NMREPRINSEST AS DSINSCESTA,
            V.NRORDPRIORIDADE,
            V.DTVALVINCFISC,
            V.NRORG
        FROM VINCULOFISREGRA V,
            EMPRESA E,
            TIPOOPERAC T,
            INSCRESTAD I
        WHERE 
            V.CDEMPRESA  = E.CDEMPRESA(+)
            AND V.IDENTRSAIDOP = T.IDENTRSAIDOP(+)
            AND V.CDTIPOOPER   = T.CDTIPOOPER(+)
            AND V.CDEMPRESA    = I.CDEMPRESA(+)
            AND V.CDINSCESTA   = I.CDINSCESTA(+)
            AND V.CDEMPRESA    = :P_CDEMPRESA
            AND V.IDENTRSAIDOP = :P_IDENTRSAIDOP
            AND V.CDTIPOOPER   = :P_CDTIPOOPER
            AND V.CDREGRAFISC  = :P_CDREGRAFISC
SQL;
        $params = array(
            "P_CDEMPRESA" => $cdempresa,
            "P_IDENTRSAIDOP" => $identrsaidop,
            "P_CDTIPOOPER" => $cdtipooper,
            "P_CDREGRAFISC" => $cdregrafisc
        );
        return ($this->entityManager->getConnection()->fetchAll($sql, $params));
        
    }
    
    public function getNewCodeRegra($nrorg)
    {
        $sql = <<<SQL
        SELECT (MAX(R.CDREGRAFISC) + 1) AS CODIGO FROM REGRAFISCAL R WHERE NRORG = :P_NRORG
SQL;

        $params = array(
            "P_NRORG" => $nrorg
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getImpostosGrupo($cdgrupoimpost, $nrorg) {
         $sql = <<<SQL
        SELECT CDGRUPIMPOSTO, CDIMPOSTO
          FROM ITAGRUPIMPOSTO
         WHERE CDGRUPIMPOSTO = :P_CDGRUPIMPOSTO
         AND   NRORG         = :P_NRORG
SQL;

        $params = array(
            "P_CDGRUPIMPOSTO" => $cdgrupoimpost,
            "P_NRORG" => $nrorg
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function saveAgrupImposto($cdRegraFisc, $cdGrupImposto) {
        $sql = "INSERT INTO IMPREGRAFISCAL (CDREGRAFISC, CDIMPOSTO, IDIMPOBRIGREGRA)
                SELECT
                   :P_CDREGRAFISC,
                   CDIMPOSTO,
                   'S'
                 FROM ITAGRUPIMPOSTO
                WHERE CDGRUPIMPOSTO = :P_CDGRUPIMPOSTO";
          $params = array(
            "P_CDGRUPIMPOSTO"   => $cdGrupImposto,
            "P_CDREGRAFISC"     => $cdRegraFisc
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getImpostosGrupoAntigo($cdregrafisc, $nrorg){
        $sql = "SELECT DISTINCT R.CDGRUPIMPOSTO, T.CDIMPOSTO 
                FROM REGRAFISCAL R 
                    JOIN ITAGRUPIMPOSTO T ON R.CDGRUPIMPOSTO = T.CDGRUPIMPOSTO 
                    JOIN IMPREGRAFISCAL P ON R.CDREGRAFISC = P.CDREGRAFISC AND P.CDIMPOSTO = T.CDIMPOSTO
                WHERE R.CDREGRAFISC = :P_CDREGRAFISC ORDER BY R.CDREGRAFISC";
                
        $params = array(
            "P_CDREGRAFISC" => $cdregrafisc
        );
        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function validaProdutoRepitidoRegra($cdregrafisc, $cdpaisdest, $sgestadest, $cdpaisorig, $sgestaorig, $idtipodestfisc, $idtipoorigfisc, $idutufdest, $idutuforig, $cdproduto, $cdimposto) {
        
        $sql = "SELECT
                    CDPRODUTO
                FROM
                    DADOSFISCPROD
                WHERE
                    (
                    (( ( CDPAISDEST = :P_CDPAISDEST  AND SGESTADEST = :P_SGESTADEST ) OR :P_IDUTUFDEST = 'N' )
                    
                    AND ( ( CDPAISORIG = :P_CDPAISORIG  AND SGESTAORIG = :P_SGESTAORIG ) OR :P_IDUTUFORIG = 'N' ))
                    
                    OR ( ( IDTIPODESTFISC = 'T' AND :P_IDTIPODESTFISC = 'E' )  AND  CDPAISORIG = :P_CDPAISORIG  AND SGESTAORIG = :P_SGESTAORIG )
                    OR ( IDTIPOORIGFISC = 'T' AND :P_IDTIPOORIGFISC = 'E' )
                    
                    OR  ( IDTIPODESTFISC = 'T' AND :P_IDTIPODESTFISC = 'E' )  
                    OR ( ( IDTIPOORIGFISC = 'T' AND :P_IDTIPOORIGFISC = 'E' ) AND  CDPAISDEST = :P_CDPAISDEST  AND SGESTADEST = :P_SGESTADEST )
                    
                    
                    OR ( IDTIPODESTFISC = 'E' AND :P_IDTIPODESTFISC = 'T' )
                    OR ( IDTIPOORIGFISC = 'E' AND :P_IDTIPOORIGFISC = 'T' )
                    
                    
                    OR ( IDTIPODESTFISC = 'T' AND :P_IDTIPODESTFISC = 'T' )
                    OR ( IDTIPOORIGFISC = 'T' AND :P_IDTIPOORIGFISC = 'T' )

                    
                    )
                    
                    AND CDREGRAFISC = :P_CDREGRAFISC
                    AND CDPRODUTO = :P_CDPRODUTO
                    AND CDIMPOSTO =:P_CDIMPOSTO";
                    
                    
                
        $params = array(
            "P_CDREGRAFISC"     => $cdregrafisc,
            "P_CDPAISDEST"      => $cdpaisdest,
            "P_SGESTADEST"      => $sgestadest,
            "P_CDPAISORIG"      => $cdpaisorig,
            "P_SGESTAORIG"      => $sgestaorig,
            "P_IDTIPODESTFISC"  => $idtipodestfisc,
            "P_IDTIPOORIGFISC"  => $idtipoorigfisc,
            "P_IDUTUFDEST"      => $idutufdest,
            "P_IDUTUFORIG"      => $idutuforig,
            "P_CDPRODUTO"       => $cdproduto,
            "P_CDIMPOSTO"       => $cdimposto
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getMaxdadosfiscprod($cdregrafisc, $cdimposto) {
        $sql = "SELECT NVL(MAX(D.NRSEQDADOSFISC), 0) + 1 AS CODIGO FROM DADOSFISCPROD D WHERE D.CDREGRAFISC = :P_CDREGRAFISC AND D.CDIMPOSTO = :P_CDIMPOSTO";
       
       $params = array(
            "P_CDREGRAFISC" => $cdregrafisc,
            "P_CDIMPOSTO"   => $cdimposto
        );
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetch()["CODIGO"];
    }


        
    public function getMaxdadosfiscncm($cdregrafisc, $cdimposto) {
        $sql = "SELECT NVL(MAX(D.NRSEQDADOSFISC), 0) + 1 AS CODIGO FROM DADOSFISCNCM D WHERE D.CDREGRAFISC = :P_CDREGRAFISC AND D.CDIMPOSTO = :P_CDIMPOSTO";
       
       $params = array(
            "P_CDREGRAFISC" => $cdregrafisc,
            "P_CDIMPOSTO"   => $cdimposto
        );
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetch()["CODIGO"];
    }
}