<?php

namespace Teknisa\DF\Modules\Relatorios\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\DF\Modules\Relatorios\Services\ManifestacaoDestinatarioRelatorioService;

class ManifestacaoDestinatarioRelatorio extends \Teknisa\DF\Controllers\TeknisaCrud
{
    const DATASOURCE_NOTAFISCAL_FORN = 'manifestacao_destinatario_relatorio_fornecedor';
    const DATASOURCE_NOTAFISCAL_CLIE = 'manifestacao_destinatario_relatorio_cliente';
    const DATASOURCE_NOTAFISCAL_FILI = 'manifestacao_destinatario_relatorio_filial';
    const DATASOURCE_DEFAULT = 'manifestacao_destinatario_relatorio';
    
    protected $environment;
    protected $maniRelatorioFactory;
    protected $dataSourceName = 'manifestacao_destinatario_relatorio';
    protected $parameterBagColumns = array(
        'IDENTRSAIDNF',
        'CDTIPOOPER',
        'CDEMPRESA',
        'IDSTATUSNFENF',
        'IDENTRSAIDO',
        'IDORIGDESTNF',
        'NRINSJURIEST',
        'CDINSCESTA',
        'IDSTATUSNFE',
        'IDSTATUSMANIF',
        'NRACESSONFE',
        'SINCRONIZACAO',
        'P_DTEMISSAOINI',
        'P_DTEMISSAOF'
    );
    
    
    public function __construct($crudService, $parameterBagColumns, $environment, ManifestacaoDestinatarioRelatorioService $maniRelatorioFactory)
    {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBagColumns;
        $this->environment = $environment;
        $this->maniRelatorioFactory = $maniRelatorioFactory;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $keyRemoveIdorigdestnf = null;
        //$this->dataSourceName = self::DATASOURCE_DEFAULT;
        $filter = array();

        foreach ($conditions as $key => $condition) {
            if($condition['columnName'] === 'DTEMISSAO'){
                $dtini = $condition['value'][0];
                $dtfim = $condition['value'][1];
            }
            
            if ($condition['columnName'] === 'IDORIGDESTNF') {
                $idorigdestnf = str_split($condition['value']);
                switch ($idorigdestnf) {
                    case in_array('F', $idorigdestnf):
                        $this->dataSourceName = self::DATASOURCE_NOTAFISCAL_FORN;
                        break;
                    case in_array('C', $idorigdestnf):
                        $this->dataSourceName = self::DATASOURCE_NOTAFISCAL_CLIE;
                        break;
                    case in_array('L', $idorigdestnf):
                        $this->dataSourceName = self::DATASOURCE_NOTAFISCAL_FILI;
                        break;
                }
                //keyRemoveIdorigdestnf = $key;
                
            }
        }
        
        /*if (is_numeric($keyRemoveIdorigdestnf))
            array_splice($conditions, $keyRemoveIdorigdestnf, 1);*/
        
        foreach ($conditions as $key => $condition) {
            if ($condition['columnName'] === 'DTEMISSAO') {
                $keyRemoveDTEMISSAO = $key;
            }
        }
        if (is_numeric($keyRemoveDTEMISSAO))
            array_splice($conditions, $keyRemoveDTEMISSAO, 1);
            
        $P_DTINI['columnName'] = "P_DTEMISSAOINI";
        $P_DTINI['value'] = $dtini;
        $P_DTINI['operator'] = '=';
        
        $P_DTFIM['columnName'] = "P_DTEMISSAOFIN";
        $P_DTFIM['value'] = $dtfim;
        $P_DTFIM['operator'] = '=';
        
        array_push($conditions, $P_DTINI);
        array_push($conditions, $P_DTFIM);
        
       
        
        foreach ($conditions as $campo){
            $valor = '';
            if(is_array($campo['value']))
                foreach($campo['value'] as $value)
                    $valor = $valor.'_'.$value;
                
                if(!empty($valor))
                    $$campo['columnName'] = $valor;
                else
                    $$campo['columnName'] = $campo['value'];
            }
        
        foreach($this->parameterBag as $parameter)
            if(!$$parameter)
                $$parameter = 'TD';

        $rows = $this->maniRelatorioFactory->getRelatorio($IDENTRSAIDNF, $CDTIPOOPER, $CDEMPRESA, $IDSTATUSNFENF, $IDENTRSAIDOP, $IDORIGDESTNF, $NRINSJURIEST, $CDINSCESTA, $IDSTATUSNFE, $IDSTATUSMANIF, $NRACESSONFE, $SINCRONIZACAO, $P_DTEMISSAOINI, $P_DTEMISSAOFIN);
        
        
        //var_dump($rowss);die;
        
        foreach ($rows as &$row) {
            $sincronizacao = 0;
            if(is_null($row['NRLANCTONFDEST'])  && !is_null($row['NRLANCTONF']) && $row['NRACESSONFE']) $sincronizacao = 0;
            else if(is_null($row['NRLANCTONFDEST']) && is_null($row['NRACESSOINTG'])) $sincronizacao = 1;
            else if($row['NRLANCTONFDEST']) $sincronizacao = 2;
            else if(is_null($row['NRLANCTONFDEST']) && $row['NRACESSOINTG']) $sincronizacao = 3;
            $row['SINCRONIZACAO'] = $sincronizacao;
        }
        
        $dataSet = new DataSet('manifestacao_destinatario_relatorio', $rows);
        $response->setDataSets(array($dataSet));
    }
}